@@ -3,6 +3,7 @@ import os.path
 import shutil
 import logging
 from wikked.commands.base import WikkedCommand, register_command
+from wikked.wiki import Wiki, WikiParameters, INIT_CONTEXT
 
 
 logger = logging.getLogger(__name__)
@@ -40,8 +41,8 @@ class InitCommand(WikkedCommand):
 
         logger.info("Initializing new wiki at: %s" % path)
         from wikked.wiki import WikiParameters, Wiki
-        parameters = WikiParameters(path)
-        wiki = Wiki(parameters, for_init=True)
+        parameters = WikiParameters(path, ctx=INIT_CONTEXT)
+        wiki = Wiki(parameters)
         wiki.init()
 
         if not ctx.args.bare:
@@ -90,7 +91,10 @@ class UpdateCommand(WikkedCommand):
                 nargs='?')
 
     def run(self, ctx):
-        ctx.wiki.updatePage(path=ctx.args.path)
+        if ctx.args.path:
+            ctx.wiki.updatePage(path=ctx.args.path)
+        else:
+            ctx.wiki.updateAll()
 
         if ctx.args.debug and ctx.args.path:
             page_info = ctx.wiki.fs.getPageInfo(ctx.args.path)
