@@ -8,6 +8,10 @@ from wikked.commands.base import WikkedCommand, register_command
 logger = logging.getLogger(__name__)
 
 
+def autoreload_wiki_updater(wiki, url):
+    wiki.db.uncachePages(except_url=url, only_required=True)
+
+
 @register_command
 class RunServerCommand(WikkedCommand):
     def __init__(self):
@@ -61,17 +65,27 @@ class RunServerCommand(WikkedCommand):
                 else:
                     setattr(wikked.settings, cname, cval)
 
+        # Remove Flask's default logging handler. Since the app is under the
+        # overall Wikked package, logging is handled by the root logger
+        # already.
+        wikked.settings.WIKI_NO_FLASK_LOGGER = True
+
+        # When running from the command line, we only have one web server
+        # so make it also serve static files.
+        wikked.settings.WIKI_SERVE_FILES = True
+        if ctx.args.dev:
+            wikked.settings.WIKI_DEV_ASSETS = True
+        if not ctx.args.noupdate:
+            wikked.settings.WIKI_AUTO_RELOAD = True
+            ctx.params.wiki_updater = autoreload_wiki_updater
+
         # Create/import the app.
         from wikked.web import app
+        app.wiki_params = ctx.params
+        ctx.wiki.db.hookupWebApp(app)
 
-        # Setup other simpler settings.
-        if ctx.args.dev:
-            app.config['DEV_ASSETS'] = True
-        if not ctx.args.noupdate:
-            app.config['WIKI_AUTO_RELOAD'] = True
-
-        app.set_wiki_params(ctx.params)
-        if bool(app.config.get('UPDATE_WIKI_ON_START')):
+        # Update if needed.
+        if bool(app.config.get('WIKI_UPDATE_ON_START')):
             ctx.wiki.updateAll()
 
         # Run!
