@@ -2,7 +2,8 @@ import os
 import os.path
 import logging
 from base import WikiIndex, HitResult
-from whoosh.analysis import StemmingAnalyzer, CharsetFilter, NgramFilter
+from whoosh.analysis import (StandardAnalyzer, StemmingAnalyzer,
+        CharsetFilter, NgramFilter)
 from whoosh.fields import Schema, ID, TEXT, STORED
 from whoosh.highlight import WholeFragmenter
 from whoosh.index import create_in, open_dir
@@ -34,7 +35,14 @@ class WhooshWikiIndex(WikiIndex):
             self._indexPage(writer, page)
         writer.commit()
 
-    def update(self, pages):
+    def updatePage(self, page):
+        logger.info("Updating index for page: %s" % page.url)
+        writer = self.ix.writer()
+        self._unindexPage(writer, page.url)
+        self._indexPage(writer, page)
+        writer.commit()
+
+    def updateAll(self, pages):
         logger.info("Updating index...")
         to_reindex = set()
         already_indexed = set()
@@ -95,8 +103,8 @@ class WhooshWikiIndex(WikiIndex):
             return hits
 
     def _getSchema(self):
-        preview_analyzer = (StemmingAnalyzer() | CharsetFilter(accent_map) |
-                NgramFilter(minsize=3))
+        preview_analyzer = (StandardAnalyzer() | CharsetFilter(accent_map) |
+                NgramFilter(minsize=1))
         text_analyzer = StemmingAnalyzer() | CharsetFilter(accent_map)
         schema = Schema(
                 url=ID(stored=True),
