@@ -30,6 +30,7 @@ class PageData(object):
     def __init__(self):
         self.url = None
         self.path = None
+        self.cache_time = None
         self.title = None
         self.raw_text = None
         self.formatted_text = None
@@ -56,6 +57,14 @@ class Page(object):
     def path(self):
         return self._data.path
 
+    @property
+    def cache_time(self):
+        return self._data.cache_time
+
+    @property
+    def is_resolved(self):
+        return self._data.is_resolved
+
     @property
     def extension(self):
         if self._data.path is None:
@@ -133,6 +142,7 @@ class FileSystemPage(Page):
         data = PageData()
         data.url = page_info.url
         data.path = page_info.path
+        data.cache_time = None
         data.raw_text = page_info.content
 
         # Format the page and get the meta properties.
