@@ -9,11 +9,12 @@ from wikked.utils import PageNotFoundError
 from wikked.views import (is_page_readable, get_page_meta, get_page_or_404,
         url_from_viewarg,
         CHECK_FOR_READ)
-from wikked.web import app
+from wikked.web import app, get_wiki
 
 
 def get_history_data(history, needs_files=False):
     hist_data = []
+    wiki = get_wiki()
     for i, rev in enumerate(reversed(history)):
         rev_data = {
             'index': i + 1,
@@ -27,9 +28,9 @@ def get_history_data(history, needs_files=False):
             rev_data['pages'] = []
             for f in rev.files:
                 url = None
-                path = os.path.join(g.wiki.root, f['path'])
+                path = os.path.join(wiki.root, f['path'])
                 try:
-                    page = g.wiki.db.getPage(path=path)
+                    page = wiki.db.getPage(path=path)
                     # Hide pages that the user can't see.
                     if not is_page_readable(page):
                         continue
@@ -45,7 +46,6 @@ def get_history_data(history, needs_files=False):
                     'action': ACTION_NAMES[f['action']]
                     })
             rev_data['num_pages'] = len(rev_data['pages'])
-            rev_data['make_collapsable'] = len(rev_data['pages']) > 1
             if len(rev_data['pages']) > 0:
                 hist_data.append(rev_data)
         else:
@@ -53,20 +53,22 @@ def get_history_data(history, needs_files=False):
     return hist_data
 
 
-@app.route('/api/history')
+@app.route('/api/site-history')
 def api_site_history():
-    limit = request.args.get('l')
-    if not limit:
-        limit = 10
-    else:
-        limit = int(limit)
-
-    history = g.wiki.getHistory(limit=limit)
+    wiki = get_wiki()
+    after_rev = request.args.get('rev')
+    history = wiki.getHistory(limit=10, after_rev=after_rev)
     hist_data = get_history_data(history, needs_files=True)
     result = {'history': hist_data}
     return jsonify(result)
 
 
+@app.route('/api/history/')
+def api_main_page_history():
+    wiki = get_wiki()
+    return api_page_history(wiki.main_page_url.lstrip('/'))
+
+
 @app.route('/api/history/<path:url>')
 def api_page_history(url):
     page = get_page_or_404(url, check_perms=CHECK_FOR_READ)
@@ -126,7 +128,8 @@ def api_revert_page(url):
             'author': author,
             'message': message
             }
-    g.wiki.revertPage(url, page_fields)
+    wiki = get_wiki()
+    wiki.revertPage(url, page_fields)
     result = {'reverted': 1}
     return jsonify(result)
 
